﻿using Infragistics.Windows.DataPresenter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace GridRowDelete.CustomizationLogic
{
    public class DataGridColumnLogic : IDisposable 
    {

          
        private XElement FindLayoutInDocument(XDocument p_Doc, string p_Name)
        {
            IEnumerable<XElement> ListLayouts = p_Doc.Element("xamDataPresenter").Element("fieldLayouts").Elements("fieldLayout");
            return ListLayouts.SingleOrDefault(p_Layout => p_Layout.Attribute("key").Value == p_Name);
        }

 
        /// <summary>
        /// Find the field in the layout.
        /// </summary>
        private XElement FindFieldInLayout(XElement p_Layout, string p_FieldName)
        { 
            return p_Layout.Element("fields").Elements("field").SingleOrDefault(p => (p_FieldName == (string)p.Attribute("name")));
        }


        /// <summary>
        /// Save the current customizations to a text-based xml format.
        /// </summary>
        public string GetCurrentBatchCustomizations(XamDataGrid p_Grid, bool p_GetGenericEquivalent)
        {
            // Get the current document
            XDocument CurrentDocumentObj = XDocument.Parse(p_Grid.SaveCustomizations());
            if(!p_GetGenericEquivalent) return CurrentDocumentObj.ToString();

            // The layouts (most of which I will ignore)
            IEnumerable<XElement> ListLayouts = CurrentDocumentObj.Element("xamDataPresenter").Element("fieldLayouts").Elements("fieldLayout");
            ListLayouts.Where(p => string.IsNullOrEmpty((string)p.Attribute("key")) ).ToList().ForEach(p_LayoutElement => p_LayoutElement.Remove());

            // Loop the three layouts
            foreach(string LoopLayout in ListLayouts.Select(p => (string)p.Attribute("key")).ToList())
            {
                // Get the layout.  Change it to a generic
                XElement LayoutElement = FindLayoutInDocument(CurrentDocumentObj, LoopLayout);
                if(LoopLayout == "TopMovieFieldLayout")
                {
                    LayoutElement.SetAttributeValue("key", "GenericFieldLayout");
                    LayoutElement.SetAttributeValue("parentFieldName", "GenericParent");
                    continue;
                }

                if(LoopLayout == "GroupFieldLayout")
                {
                    continue;
                }

                // Delete everything else
                LayoutElement.Remove();
            }

            // Return it
            return CurrentDocumentObj.ToString();
        }
        
        public string MakeHidden(string p_ColumnName, string p_Input)
        {
            // Now load the current version of grid layout.  
            // The current version needs to be modified with desired changes.
            // Unfortunately we can't just load the desired document itself.  We need some merging...
            XDocument CurrentDocumentObj = XDocument.Parse(p_Input);
            foreach(string LoopLayout in new string[] {"GenericFieldLayout"} )
            {
                // Get the layout and fields to use
                XElement CurrentLayoutElement = FindLayoutInDocument(CurrentDocumentObj, LoopLayout);

                // Really bad
                if(CurrentLayoutElement == null)
                {  throw new InvalidOperationException(LoopLayout + " was not found in the list of active current layouts."); }

                // Update fields.  Use the field name corresponding to this layout
                foreach(string LoopDesiredFieldName in CurrentLayoutElement.Element("fields").Elements("field").Select(p => (string)p.Attribute("name")))
                {
                    if(p_ColumnName != LoopDesiredFieldName) continue;
                    // Get the two fields (one as desired, one to which we will apply changes.)
                    XElement LoopApplyField = FindFieldInLayout(CurrentLayoutElement, LoopDesiredFieldName);
                     
                    // Set the attribute to the desired value
                    LoopApplyField.SetAttributeValue("Visibility", "Hidden"); 


                } // desired field

            } // layout


            return CurrentDocumentObj.ToString();

        }
        
        /// <summary>
        /// Move the column
        /// </summary>
        public string MoveColumn(string p_ColumnName, int p_ColumnPos, string p_Input)
        {
            // Now load the current version of grid layout.  
            // The current version needs to be modified with desired changes.
            // Unfortunately we can't just load the desired document itself.  We need some merging...
            XDocument CurrentDocumentObj = XDocument.Parse(p_Input);
            foreach(string LoopLayout in new string[] { "GenericFieldLayout" } )
            {
                // Get the layout and fields to use
                XElement CurrentLayoutElement = FindLayoutInDocument(CurrentDocumentObj, LoopLayout);

                // Really bad
                if(CurrentLayoutElement == null)
                {  throw new InvalidOperationException(LoopLayout + " was not found in the list of active current layouts."); }
                int defaultPosition = 0;
                int oldPosition = defaultPosition;

                // Update fields.  Use the field name corresponding to this layout
                foreach (string LoopDesiredFieldName in CurrentLayoutElement.Element("fields").Elements("field").Select(p => (string)p.Attribute("name")))
                {

                    // Get the two fields (one as desired, one to which we will apply changes.)
                    XElement LoopApplyField = FindFieldInLayout(CurrentLayoutElement, LoopDesiredFieldName);
                    if (p_ColumnName != LoopDesiredFieldName )
                    {

                        if (LoopApplyField.Attribute("column") == null)
                        {
                            if (p_ColumnPos == defaultPosition)
                                defaultPosition++;
                            // Get the two fields (one as desired, one to which we will apply changes.)
                            LoopApplyField = FindFieldInLayout(CurrentLayoutElement, LoopDesiredFieldName);

                            LoopApplyField.SetAttributeValue("column", defaultPosition.ToString());
                            LoopApplyField.SetAttributeValue("row", "0");
                            LoopApplyField.SetAttributeValue("rowSpan", "1");
                            LoopApplyField.SetAttributeValue("columnSpan", "1");
                            LoopApplyField.SetAttributeValue("isCollapsedInLayout", "false");
                            defaultPosition++;
                        }
                    }
                    else
                    {
                        oldPosition = defaultPosition;
                        LoopApplyField = FindFieldInLayout(CurrentLayoutElement, LoopDesiredFieldName);
                        
                        // Set the attribute to the desired value
                        LoopApplyField.SetAttributeValue("column", p_ColumnPos.ToString());
                        LoopApplyField.SetAttributeValue("row", "0");
                        LoopApplyField.SetAttributeValue("rowSpan", "1");
                        LoopApplyField.SetAttributeValue("columnSpan", "1");
                        LoopApplyField.SetAttributeValue("isCollapsedInLayout", "false");

                        defaultPosition = oldPosition;
                    }
          
                } // desired field

            } // layout
            return CurrentDocumentObj.ToString();

        }
        
        public void LoadCustomizations(XamDataGrid p_DataGrid, string p_FieldLayoutNameToLoad, string p_ParentRelationNameToLoad, string p_Input)
        {
            // Now load the current version of grid layout.  
            // The current version needs to be modified with desired changes.
            // Unfortunately we can't just load the desired document itself.  We need some merging...
            XDocument CurrentDocumentObj = XDocument.Parse(p_Input);
            foreach (string LoopLayout in new string[] { "GenericFieldLayout" })
            {
                // Get the layout and fields to use
                XElement CurrentLayoutElement = FindLayoutInDocument(CurrentDocumentObj, LoopLayout);

                CurrentLayoutElement.SetAttributeValue("key", p_FieldLayoutNameToLoad);
                CurrentLayoutElement.SetAttributeValue("parentFieldName", p_ParentRelationNameToLoad);

            }
            // Load the customized version of the xml back into the xamDataGrid
            p_DataGrid.LoadCustomizations(CurrentDocumentObj.ToString());            
        }

        public void Dispose()
        {
        }
    }
}
